## Intruduction

Puremvc-Express module is a bootstrap kit to help you to build backend service.
Especially for developers who are familiar with Puremvc and Express.

## Usage

### Initial 
```javascript
/** index.js */
const fs = require('fs');
const winston = require('winston');
let { CORE_EVENTS, App } = require('puremvc');
let options = require('./options');
let app = new App('test', options);

app.start();
```

```javascript
/** options.js */
const options = {
    httpServer: {
        port: 3000,
        sslPort: 3002,
        credentials: {
            key: fs.readFileSync('./ssl/key.pem', 'utf8'),
            cert: fs.readFileSync('./ssl/cert.pem', 'utf8')
        }
    },
    database: {
        host: 'database.your.domain',
        user: 'dbuser',
        password: 'dbpassword',
        database: 'dbname',
        connectionLimit: 2
    },
    memCache: {
        host: 'redis.your.domain',
        port: 6379
    },
    firebase: {
        apiKey: 'yourFirebaseApiKey',
        authDomain: 'yourAuthDomain.firebaseapp.com',
        databaseURL: 'https://yourFirebaseProjectId.firebaseio.com',
        projectId: 'yourFirebaseProjectId',
        storageBucket: 'yourFirebaseProjectId.appspot.com',
        messagingSenderId: '1234567890',
        appId: '1:1234567890:web:0123456789abcdef',
        measurementId: 'G-0123456789'
    },
    passport: {
        jwt: {
            secret: 'yourJwtSecret',
            signOptions: {
                algorithm: 'algorithm-you-want',
                expiresIn: '900s'
            }
        },
        local: {
            usernameField: 'username',
            passwordField: 'password'
        },
        facebook: {
            clientID: 'facebookID',
            secret: 'facebookSecret',
            callbackURL: 'https://localhost:3000/auth/facebook/callback'
        },
        twitter: {
            consumerKey: 'twitterKey',
            consumerSecret: '',
            callbackURL: ''
        },
        google: {
            consumerKey: 'googleKey',
            consumerSecret: '',
            callbackURL: ''
        }
    },
    session: {
        secret: 'some secret',
        resave: false,
        saveUninitialized: false
    },
    di: {
        m_appData: 'CORE:APP-DATA',
        m_httpServer: 'CORE:HTTP-SERVER',
        m_database: 'CORE:DATABASE',
        m_passport: 'CORE:PASSPORT',
        m_memCache: 'CORE:MEM-CACHE',
        m_logger: 'CORE:LOGGER',
        m_users: 'CORE:USERS'
    },
    logger: {
        level: 'info',
        transports: [
            new winston.transports.Console(),
            new winston.transports.File({ filename: 'logs/app.log' })
        ]
    },
    users: [
        { id: 1, username: 'root', password: 'p4s5w0rD', role: 'admin' },
        { id: 2, username: 'trumpjojo', password: '9453', role: 'author' },
        { id: 3, username: 'ingrid', password: '0908', role: 'people\'s gf' },
        { id: 4, username: 'dora', password: '9ido', role: 'amon' }
    ]
}

export default options;
```

### Dependency Injection

This feature supports inject object which extends {puremvc.Proxy}
For example

```Javascript
import { Mediator } from 'puremvc';
import { CORE_EVENTS } from 'puremvc-express';

export default class MyMediator extends Mediator {
    m_logger = undefined;
    logger;
    constructor () {
        super('NAME')
    }

    onRegister () {
        this.sendNotification(CORE_EVENTS.DI, this);
        // Now you can access this.m_logger
    }
}
```

Object injected depends on values under `di` of options, corresponding to the string assigned to super method.  This feature use retrieveProxy in puremvc class to approach.
```Javascript
    consturctor () {
        super('NAME USE TO INJECT');
    }
```

### Logger

Puremvc-Express uses Winston as logging tool, we wrap it to similar as Log4js.
By following example, you can learn how to use it.

```Javascript
class YourClass extends SimpleCommand {
    m_logger = undefined;
    logger;

    constructor () {
        super('NAME');
    }

    execute (notification) {
        this.sendNotification(CORE_EVENTS.DI, this);
        this.logger = this.m_logger.getLogger(this.constructor.name);
        this.logger.info('Hello World');
        //2021-03-02 13:44:24.643 info [YourClass]: Hello World
    }
}
```

### Register your customized Proxy, Mediator, Command

After puremvc-express was constructed, you can access its facade member then register following three commnand to prepare your `Proxy`, `Mediator` and `Command`.

```Javascript
let { CORE_EVENTS, App } = require('puremvc-express');

let app = new App('AppName');
let facade = app.facade;

facade.registerCommand(CORE_EVENTS.INIT_COMMANDS, YourCommandRegistor);
facade.registerCommand(CORE_EVENTS.INIT_MEDIATORS, YourMediatorRegistor);
facade.registerCommand(CORE_EVENTS.INIT_PROXIES, YourProxyRegistor);

app.options = {...options};
app.start();
```

### Serving static files

Puremvc-express defaults serve 'public' folder 

### Local stretagy of Passport 

In order to maping to you user data verifing, please register 

```Javascript
class UserVerify extends SimpleCommand {
    m_database = undefined;
    m_logger = undefined;
    logger;

    constructor() {
        super()
    };

    async execute(notification) {
        this.sendNotification(CORE_EVENTS.DI, this);
        this.logger = this.m_logger.getLogger('UserVerify');
        let { username, password, done } = notification.body;
        let result = await this.m_database.query(
            `SELECT * from users where username like "${username}";`
        );

        let hit = await result.filter((obj, idx, ary) => {
            return obj.loginID == username;
        })
        if (hit.length && await bcrypt.compare(password, hit[0].password)) {
            done(null, hit[0]);
        } else
            done(null, false);

    }
}

facade.registerCommand(CORE_EVENTS.USER_VERIFY, UserVerify);
```

### Authenticate required path

Puremvc-Express default provides bearer token checking mechanism for every request paths under `http://localhost:3000/auth/...`, the request header will need bearer token `curl -H 'authorization: bearer <token>' ...`

### Signing bearer token


### Simple Users

This kit provides you a simple way to setup your user list, 

### Included API's

#### Sign in
This API sends notifcation `CORE_EVENTS.AUTH_SIGNIN` with parameter `req`, `res`, `next`.
If there has the event name registered.
- Path: `/auth/signin`
- Method: `POST`

#### Sign up
This API sends notification `CORE_EVENTS.AUTH_SIGNUP` with parameter `req`, `res`, `next`.
If there has the event name registered.
- Path: `/auth/signup`
- Method: `POST`

#### Sign JWT
- Path: `/auth/signjwt`
- Method: `POST`
- Header: `content-type: application/json`
- Body:
    ```json
    {
        username: 'user', 
        password: 'pwd', 
        payload: {
            your:'data'
        }
    }
    ```
- Return: Bearer token using payload object.
- CURL:
    ```bash
    curl -H "content-type: application/json" -d '{"username":"root","password":"p4s5w0rD","payload":{"your":"data"}}' localhost:3000/auth/signjwt
    ```
#### Echo message
- Path: `/echo/:message`
- Method: `ALL`
- Return: message
- CURL:
    ```bash
    curl localhost:3000/echo/hello%20world
    ```
#### Authorized echo message 
- Path: `/api/echo/:message`
- Method: `ALL`
- Header: `authorization: bearer <token>`
- Return: message
- CURL: 
    ```bash
    curl -H "authorization: bearer <token>" localhost:3000/api/echo/hello%20world
    ```
